#include "ti86asm.inc"

IntAddress equ $8f8f

.org _asm_exec_ram
 
 call StartIm2          ; Starts IM2
 
 call _flushallmenus
 
 res appTextSave,(IY+appflags) 
 res appAutoScroll,(IY+appflags)
 
 ld hl,$fc00       ; Clears screen
 ld (hl),0
 ld de,$fc01
 ld bc,1023
 ldir
 
 ld hl,$0000          ; Displays briandm03@yahoo.com
 ld (_curRow),hl
 ld hl,BrianStr
 call _puts

 call EnterKeyLoop  ; Waits for enter/exit key to be pressed
 
 ld hl,$fc00       ; Clears screen
 ld (hl),0
 ld de,$fc01
 ld bc,1023
 ldir 
 
 ld hl,Banner           ; Displays title
 ld de,$fc50
 ld bc,256
 ldir
 
 ld hl,$242a            ; Displays press enter
 ld (_penCol),hl
 ld hl,PressEnterStr
 call _vputs 
 
 ld hl,PointyMoon       ; Draws moon
 call DrawMoon

 ld a,0                 ; Sets variables
 ld (FlameDrawn),a
 ld a,25
 ld (ShipY),a
 ld a,127
 ld (ShipX),a
 ld hl,$1fff
 ld (VValue),hl
 ld hl,$03ff
 ld (HValue),hl
 ld a,10
 ld (FrameTimer),a
 
 ld de,(ShipX)          ; Draws ship
 ld hl,$fc00
 ld ix,Lander
 ld bc,Background
 call UCSR

 call Delay
 ld a,255
 ld (KeyFlag),a 
TitleLoop:              ; loop that sends ship across screen,
 ld a,(KeyFlag)         ; flashes press enter, checks for enter/exit key
 bit 4,a
 jp z,Initial
 bit 6,a
 jp z,Exit
 ld hl,(VValue)
 ld de,1
 sbc hl,de
 ld (VValue),hl
 jp nc,SkipFlash
 ld hl,$1fff
 ld (VValue),hl
 bit textInverse, (iy+textflags)
 jp z,SetInvert
 res textInverse, (iy+textflags)
 jp InvertDone
SetInvert:
 set textInverse, (iy+textflags)
InvertDone:
 ld hl,$242a
 ld (_penCol), hl
 ld hl,PressEnterStr
 call _vputs 
SkipFlash:
FSL:
 ld hl,(HValue)
 ld de,1
 sbc hl,de
 ld (HValue),hl
 jp nc,SkipShip
 ld hl,$03ff
 ld (HValue),hl
 ld a,(ShipX)
 cp 250
 call z,SSL
 dec a
 ld (ShipX),a
 call DrawShip
 ld a,(ShipX)
 add a,8
 cp 128
 jp nc,SkipShip
 ld e,a
 ld a,(ShipY)
 ld d,a
 ld ix,FlameLeft1
 call DrawFlame 
SkipShip:
 jp TitleLoop
 
SSL:
 ld a,128
 ld (ShipX),a 
 ret





Initial:        ; Sets Lives and Level when program first runs
 ld a,3
 ld (Lives),a
 ld a,1
 ld (Level),a
 res textInverse, (iy+textflags) 


Level1:
 ld hl,50           ; Sets level specific variables
 ld (Gravity),hl
 ld hl,Level1InfoStr
 ld bc,FlatMoon
 push bc
 push hl
 jp Default 
 
Level2:
 ld hl,100           ; Sets level specific variables
 ld (Gravity),hl
 ld hl,Level2InfoStr
 ld bc,RockyMoon
 push bc
 push hl
 jp Default 

Level3:
 ld hl,150           ; Sets level specific variables
 ld (Gravity),hl
 ld hl,Level3InfoStr
 ld bc,PointyMoon
 push bc
 push hl
 jp Default 

Level4:
 ld hl,200           ; Sets level specific variables
 ld (Gravity),hl
; ld a,0
; ld (LevelType),a
 ld hl,Level4InfoStr
 ld bc,TrenchMoon
 push bc
 push hl
 jp Default 


Level5:
 ld hl,250           ; Sets level specific variables
 ld (Gravity),hl
 ld hl,Level5InfoStr
 ld bc,CaveMoon
 push bc
 push hl
 jp Default 

Level6:
 ld hl,300           ; Sets level specific variables
 ld (Gravity),hl
 ld hl,Level6InfoStr
 ld bc,WaveMoon
 push bc
 push hl
 jp Default  ; awesome

                      
Default:
 ld hl,$7fff    ; Sets default variables for every level
 ld (VTimer),hl
 ld (HTimer),hl
 ld (GTimer),hl
 ld hl,0
 ld (VValue),hl
 ld (HValue),hl
 ld hl,$147f
 ld (Fuel),hl
 ld a,0
 ld (VDirection),a
 ld (HDirection),a
 ld (FlameDrawn),a
 ld (KeyFlag),a
 ld (Speed),a
 ld (FuelEmpty),a
 ld a,10
 ld (FrameTimer),a

 ld hl,$fc00       ; Clears screen
 ld (hl),0
 ld de,$fc01
 ld bc,1023
 ldir
 
 ld hl,$0000          ; Displays level info
 ld (_curRow),hl
 pop hl
 call _puts

 call EnterKeyLoop  ; Waits for enter/exit key to be pressed
 
 ld hl,$fc00       ; Clears screen
 ld (hl),0
 ld de,$fc01
 ld bc,1023
 ldir 

 pop hl ;whoa
 call DrawMoon 

 ld a,8    ; sets ship position for moon level
 ld (ShipY),a
 ld a,60
 ld (ShipX),a

 call DrawStats ; Draws stats and updates them
 call UpdateLives
 call UpdateSpeed
 call UpdateGravity

 ld de,(ShipX)      ; Draws ship
 ld hl,$fc00
 ld ix,Lander
 ld bc,Background
 call UCSR

Idle:               ; Responds to key presses, updates timers
 ld a,(KeyFlag)
 cp 255
 jp z,NoKey
 ld b,a
 ld a,255
 ld (KeyFlag),a
 bit 6,b
 jp z,Exit
 call ClearFlame
 ld a,(FuelEmpty)
 cp 0
 jp nz,DoneKey  
NextKey1:
 bit 3, b
 jp nz, NextKey2
 call Up
 jp DoneKey
NextKey2:
 bit 1, b
 jp nz, NextKey3
 call Left
 jp DoneKey
NextKey3:
 bit 2, b
 jp nz, NextKey4
 call Right
 jp DoneKey
NextKey4:
 bit 0, b
 jp nz, DoneKey
 call Down
DoneKey:
SpeedAdjust:    ; main
 ld hl,(VValue)
 ex de,hl
 ld hl,(HValue)
 srl h
 rr l 
 add hl,de
 srl h
 rr l
 ld a,l
 cp 100
 jp c,SpeedGood
 ld a,99
SpeedGood:
 ld (Speed),a
 call _unpack_hex ; hex in a -> c first digit, b second digit decimal
 ld a,c
 add a,$30
 ld hl,$001c
 ld (_penCol), hl
 call _vputmap
 ld a,b
 add a,$30
 call _vputmap
NoKey:
 ld hl,(HValue)
 ex de,hl
 ld hl,(HTimer)
 sbc hl,de
 ld (HTimer),hl
 jp nc,NextTimer1
HTimerExpired:          ; Changes ships horizontal position
 ld a,(HDirection)
 cp 0
 jp z,ShipLeft
ShipRight:
 ld a,(ShipX)
 cp 127
 jp z,FlewOffCourse
 inc a
 ld (ShipX),a
 call DrawShip
 call DecHValue
 jp DoneShipH
ShipLeft:
 ld a,(ShipX)
 cp 250
 jp z,FlewOffCourse
 dec a
 ld (ShipX),a
 call DrawShip
 call DecHValue
DoneShipH:
 ld hl,$7fff
 ld (HTimer),hl
NextTimer1:
 ld hl,(VValue)
 ex de,hl
 ld hl,(VTimer)
 sbc hl,de
 ld (VTimer),hl
 jp nc,MoonLevel
VTimerExpired:          ; Changes ships vertical position
 ld a,(VDirection)
 cp 0
 jp z,ShipUp
ShipDown:
 ld a,(ShipY)
 cp 63
 jp z,FlewOffCourse
 inc a
 ld (ShipY),a
 call DrawShip
 call DecVValue
 jp DoneShipV
ShipUp:
 ld a,(ShipY)
 cp 1
 jp z,FlewOffCourse
 dec a
 ld (ShipY),a
 call DrawShip
 call DecVValue
DoneShipV:
 ld hl,$7fff
 ld (VTimer),hl

MoonLevel:
 ld hl,(Gravity)
 ex de,hl
 ld hl,(GTimer)
 sbc hl,de
 ld (GTimer),hl
 jp nc,Idle
GravityEffect:
 ld a,(VDirection)
 cp 255
 jp z,IncValueGravity
 call DecVValue
 jp Idle
IncValueGravity:
 call IncVValue
 jp Idle
 

 

 


DrawShip:
 call ClearFlame
 ld ix,Background
 call DrawBackground
SkipClearShip:
 ld de,(ShipX)
 ld hl,$fc00
 ld ix,Lander
 ld bc,Background
 call UCSR
 ld ix,Background
CheckCollision:
 ld h,(ix)
 inc ix
 ld l,(ix)
 inc ix
 ld c,(ix)
 ld e,c
 inc ix
 ld d,(ix)
 inc ix
CC:
 ld a,(ix)
 ld b,a
 inc ix
 ld a,(ix)
 cp b
 jp nz,Collision
 inc ix
 ld a,(ix)
 ld b,a
 inc ix
 ld a,(ix)
 cp b
 jp nz,Collision
 inc ix
 dec c
 jp nz,CC
 ld a,(ShipX)   ; ship x has to be >253 and <130 to test
 cp 130
 jp c,TestPixels
 cp 254
 ret c
TestPixels:     ; tests underneath ship to see if landed
 ld a,d
 ld d,%00101000
 cp 0
 ld b,a
 ld a,e
 ld e,0 
 jp z,SR ; zero means no rotating
Rot:
 rr d            ; sprite is rotated through de
 rr e
 djnz Rot     ; keep rotating until finished
SR:
 ld bc,16
MoveHL:
 add hl,bc
 dec a
 jp nz,MoveHL   ; hl points to right under lander
 ld a,(hl)
 and d
 ld b,a
 inc hl
 ld a,(hl)
 and e
 ld c,a ; hl == de landed else not landed
 ld h,0
 ld a,b
 cp d
 ret nz
 ld a,c
 cp e
 ret nz
 ld a,(Speed)
 cp 21
 ret nc
 pop af ; should pop whatever called it off the stack.
 jp LevelFinished




DrawFlame:
 ld bc,16
 ld a,(Frame)
 cp 0
 jp z,Frame1
 add ix,bc
Frame1:
 ld hl,$fc00
 ld bc,FlameBG
 call UCSR
 ld a,1
 ld (FlameDrawn),a
 ret

ClearFlame:
 ld a,(FlameDrawn)
 cp 0
 ret z
 ld ix,FlameBG
 call DrawBackground  
 xor a
 ld (FlameDrawn),a
 ret


 


IncVValue:
 ld hl,(VValue)
 ld a,h
 cp $80
 jp z,DecVValue
 inc hl
 ld (VValue),hl
 ret
 
DecVValue:
 or a
 ld de,1
 ld hl,(VValue)
 sbc hl,de
 ld (VValue),hl
 ret nc
VSignChange:
 ld hl,$0001
 ld (VValue),hl
 ld hl,$7fff
 ld (VTimer),hl
 ld a,(VDirection)
 xor 255
 ld (VDirection),a
 ret
 
IncHValue:
 ld hl,(HValue)
 ld a,h
 cp $80
 jp z,DecHValue
 inc hl
 ld (HValue),hl
 ret
 
DecHValue:
 or a
 ld de,1
 ld hl,(HValue)
 sbc hl,de
 ld (HValue),hl
 ret nc
HSignChange:
 ld hl,$0001
 ld (HValue),hl
 ld hl,$7fff
 ld (HTimer),hl
 ld a,(HDirection)
 xor 255
 ld (HDirection),a
 ret
 

Up:
 call DecrementFuel
 ld a,(ShipY)
 add a,8
 cp 64
 jp nc,FlameSkipUp
 ld d,a
 ld a,(ShipX)
 ld e,a
 ld ix,FlameUp1
 call DrawFlame
FlameSkipUp:
 ld a,(VDirection)
 cp 0
 jp z,IncValueUp
 call DecVValue
 ret
IncValueUp:
 call IncVValue
 ret
Down:
 call DecrementFuel
 ld a,(ShipY)
 sub 8
 cp 1
 jp c,FlameSkipDown
 ld d,a
 ld a,(ShipX)
 ld e,a
 ld ix,FlameDown1
 call DrawFlame
FlameSkipDown:
 ld a,(VDirection)
 cp 255
 jp z,IncValueDown
 call DecVValue
 ret
IncValueDown:
 call IncVValue
 ret
Left:
 call DecrementFuel
 ld a,(ShipX)
 add a,8
 cp 128
 jp nc,FlameSkipLeft
 ld e,a
 ld a,(ShipY)
 ld d,a
 ld ix,FlameLeft1
 call DrawFlame
FlameSkipLeft:
 ld a,(HDirection)
 cp 0
 jp z,IncValueLeft
 call DecHValue
 ret
IncValueLeft:
 call IncHValue
 ret
Right:
 call DecrementFuel
 ld a,(ShipX)
 sub 8
 cp 128
 jp c,DrawFlameRight
 cp 249
 jp c,FlameSkipRight
DrawFlameRight:
 ld e,a
 ld a,(ShipY)
 ld d,a
 ld ix,FlameRight1
 call DrawFlame
FlameSkipRight:
 ld a,(HDirection)
 cp 255
 jp z,IncValueRight
 call DecHValue
 ret
IncValueRight:
 call IncHValue
 ret
 
Delay:
 ld hl,$7fff
Dstart:
 nop
 dec hl  
 ld a,h
 or l
 jp nz, Dstart
 ret

UCSR:
 di
 push iy
 push bc
 pop iy
 push hl
 push de
ClipLeft:
 ld a,e
 cp 128
 jp c,ClipRight     ; < 128 clip right
 ld l,0
 neg
 ld b,a
 ld a,17
 sub b
 ld d,a
 ld e,$ff
ClipLeftMask:
 srl e      ; set clip mask
 djnz ClipLeftMask
 jp DoneX   ; finished x
ClipRight:
 ld l,a
 and 7
 ld d,a     ; set rotations
 ld b,a
 ld a,e
 ld e,$ff
 cp 121
 jp c,DoneX ; < 121 no clip
ClipRightMask:
 sla e      ; set clip mask
 djnz ClipRightMask
DoneX:
 pop bc
ClipTop:
 ld c,8
 ld a,b
 cp 8
 jp nc,ClipBottom    ; >= 8 clip bottom
 ld h,8
 ld c,a 
 xor 248
 neg
 and 7
 ld b,a
ClipTopPointer:
 inc ix
 djnz ClipTopPointer
 jp DoneY
ClipBottom:
 ld h,a
 cp 57
 jp c,DoneY ; < 57 no clip
 neg
 and 7
 ld c,a
DoneY:
 ex de,hl
 ex (sp),hl
 rl e            ; dont want to lose bit 4
 ld b,4          ; set up counter
Loop:
 srl d            ; rotate bits so that  
 rr e            ; de = y*16+x add that
 djnz Loop       ; to hl and it equals
 add hl,de       ; location to copy sprite
 pop de
 ld (iy),h
 inc iy
 ld (iy),l
 inc iy
 ld (iy),c
 inc iy
 ld (iy),d;XXXXXXXXX
 inc iy;XXXXXXXXX
LoopCopy:
 push de    ; save number of rotations and clip mask
 push bc    ; save line count
 push hl
 ld b,d     ; d = number of rotations
 ld a,(ix)       ; a = line of sprite
 and e      ; clip a
 ld d,a     ; put a into d
 ld a,(ix+8)    ; a = mask
 and e
 cpl        ; switch it
 ld h,a     ; h = finished mask
 ld e,0          ; clear e
 ld a,b          ; b = number of rotations
 ld l,$ff
 cp 0
 jr z,SkipRotate ; zero means no rotating
 ld a,b
Rotate:
 rr d            ; sprite is rotated through de
 rr e
 djnz Rotate     ; keep rotating until finished
 ld b,a     ; a = number of rotations
 scf 
RotateMask:
 rr h       ; mask is rotated through hl
 rr l
 djnz RotateMask
SkipRotate:
 ex (sp),hl
 pop bc     ; mask put in bc
 ld a,(hl)  ; gets background into a
 ld (iy),a  ; stores background at iy
 inc iy;XXXXXXXXXXXXX
 and b      ; masks background
 ld (iy),a;XXXXXXXXXXXXXX
 or d       ; puts sprite on background
 ld (hl),a  ; puts background back
 inc hl
 inc iy
 ld a,(hl)
 ld (iy),a
 inc iy;XXXXXXXXXXXXX 
 and c
 ld (iy),a;XXXXXXXXXXXXXX 
 or e
 ld (hl),a
 inc ix          ; next line of sprite
 inc iy
 ld de,15        ; y byte offset
 add hl,de       ; hl = next line in video mem
 pop bc
 pop de     ; restore rotations and mask
 dec c           ; decrement line count
 jr nz,LoopCopy
 pop iy
 ei
 ret
DrawBackground:
 ld h,(ix)
 inc ix
 ld l,(ix)
 inc ix
 ld c,(ix)
 inc ix
 inc ix;XXXXXXXXXX
 ld de,15
Draw:
 ld a,(ix)
 ld (hl),a
 inc ix
 inc ix;XXXXXXXXXXXX
 inc hl
 ld a,(ix)
 ld (hl),a
 inc ix
 inc ix;XXXXXXXXXXXXX
 add hl,de
 dec c
 jp nz,Draw
 ret

DrawStats:
 ld hl,Stats
 ld de,$fc00
 ld bc,128
 ldir
 ret

UpdateLives:
 ld a,(Lives)       ; lives = 0-3
 add a,$30
 ld hl,$0007
 ld (_penCol), hl
 call _vputmap
 ret

UpdateSpeed:
 ld a,(Speed)       ; speed = 0-99
 call _unpack_hex ; hex in a -> c first digit, b second digit decimal
 ld a,c
 add a,$30
 ld hl,$001c
 ld (_penCol), hl
 call _vputmap
 ld a,b
 add a,$30
 call _vputmap
 ret

UpdateGravity:
 ld hl,(Gravity)       ; Gravity = 0-999
 call _divHLby10
 push af
 ld a,l
 call _unpack_hex ; hex in a -> c first digit, b second digit decimal
 ld a,c
 add a,$30
 ld hl,$0049
 ld (_penCol), hl
 call _vputmap
 ld a,b
 add a,$30
 call _vputmap
 pop af
 add a,$30
 call _vputmap
 ret
 
DecrementFuel:  ; main
 ld hl,(Fuel)
 dec l
 ld (Fuel),hl
 ret nz
 push hl
 ld a,h
 add a,105
 ld e,a
 ld d,2
ClearPixels:
 ld hl,$fc00
 and 7           ; a = last three bits of e (x)
 ld c,127
 jp z,SkipClear
ClearLoop:
 scf
 rr c
 dec a
 jp nz,ClearLoop
SkipClear:
 rl e            ; dont want to lose bit 4
 srl d            ; rotate bits so that  
 rr e            ; de = y*16+x add that
 srl d            ; rotate bits so that  
 rr e            ; de = y*16+x add that
 srl d            ; rotate bits so that  
 rr e            ; de = y*16+x add that
 srl d            ; rotate bits so that  
 rr e            ; de = y*16+x add that  
 add hl,de       ; location to copy sprite
 ld de,16
 ld a,(hl)
 and c      ; c has to equal bitmask ex x =0 bitmask = %01111111 (1,%10111111)
 ld (hl),a
 add hl,de
 ld a,(hl)
 and c      ; c has to equal bitmask ex x =0 bitmask = %01111111 (1,%10111111)
 ld (hl),a
 add hl,de
 ld a,(hl)
 and c      ; c has to equal bitmask ex x =0 bitmask = %01111111 (1,%10111111)
 ld (hl),a
 add hl,de  
 pop hl
 dec h
 jp nz,FuelNotEmpty
 ld a,255
 ld (FuelEmpty),a
 ret 
FuelNotEmpty: 
 ld l,$ff
 ld (Fuel),hl
 ret  
 


FlewOffCourse:
 call ClearFlame
 ld ix,Background
 call DrawBackground
 ld hl,$1e20
 ld (_penCol), hl
 ld hl,OffCourseStr
 call _vputs
 call EnterKeyLoop
 ld a,(Lives)
 dec a
 ld (Lives),a
 jp z,Lose 
 jp PickLevel









DrawMoon:   ; hl points to moon to draw
 ld de,$ff00
 ld bc,256
 ldir
 ret 

EnterKeyLoop:
 call Delay
 ld a,255
 ld (KeyFlag),a
EKL:
 ld a,(KeyFlag)
 bit 4,a
 ret z
 bit 6,a
 jp z,Exit
 jp EKL

Exit:
 im 1
 set appAutoScroll,(IY+appflags) 
 set appTextSave,(IY+appflags)
 res textInverse, (iy+textflags)
 call _clrScrn
 call _homeup
 call _dispDone
 jp _jforcecmdnochar

LevelFinished:
 ld hl,$1e2c            ; Displays congratulations
 ld (_penCol), hl
 ld hl,LevelDoneStr
 call _vputs
 call EnterKeyLoop
 ld a,(Level)
 inc a
 ld (Level),a
 jp PickLevel
 
PickLevel:
 ld a,(Level)
 cp 1
 jp z,Level1
 cp 2
 jp z,Level2
 cp 3
 jp z,Level3
 cp 4
 jp z,Level4
 cp 5
 jp z,Level5
 cp 6
 jp z,Level6
 cp 7
 jp z,EndGame

Lose:
 call UpdateLives
 ld hl,$1e20            ; Displays loser
 ld (_penCol), hl
 ld hl,LostStr
 call _vputs
 call EnterKeyLoop
 jp Exit

EndGame:
 ld hl,$fc00       ; Clears screen
 ld (hl),0
 ld de,$fc01
 ld bc,1023
 ldir
 
 ld hl,$0000          ; Displays level info
 ld (_curRow),hl
 ld hl,EndGameStr
 call _puts

 call EnterKeyLoop  ; Waits for enter/exit key to be pressed
 jp Exit 

Collision:
 call ClearFlame
 call Explode
 ld hl,$1e2c
 ld (_penCol), hl
 ld hl,BlewUpStr
 call _vputs
 call EnterKeyLoop
 ld a,(Lives)
 dec a
 ld (Lives),a
 jp z,Lose 
 jp PickLevel
 
Explode:
 ld a,1
 ld (FrameTimer),a
 ld b,255 
ExplodeLoop:
 push bc
 ld ix,Explosion1
 ld de,(ShipX)
 ld bc,16
 ld a,(Frame)
 cp 0
 jp z,FirstFrame
 add ix,bc
FirstFrame:
 ld hl,$fc00
 ld bc,Background
 push ix
 push hl
 push de
 push bc
 ld ix,Background
 call DrawBackground 
 pop bc
 pop de
 pop hl
 pop ix
 call UCSR
 pop bc
 djnz ExplodeLoop
 ld ix,Background
 call DrawBackground  
 ret

StartIm2:
 ld hl,$8e00            ; Sets up 256 byte vector
 ld de,$8e01
 ld (hl),$8f
 ld bc, 256
 ldir
 ld hl,IntCopy          ; Copies Interrupt
 ld de,IntAddress
 ld bc,IntEnd-IntStart
 ldir
 ld a,$8e           ; Set i to jump to vector table
 ld i,a
 im 2               ; Set interrupt mode 2
 ret

Background:     ; 36 bytes
.db 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
.db 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
FlameBG:        ; 36 bytes
.db 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
.db 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
ShipX:
.db 0
ShipY:
.db 0
VTimer:
.word 0
VValue:
.word 0
HTimer:
.word 0
HValue:
.word 0
GTimer:
.word 0
Gravity:
.word 0
Fuel:
.word 0
Lives:
.db 0
Speed:
.db 0
VDirection:
.db 0
HDirection:
.db 0
Frame:
.db 0
FrameTimer:
.db 0
FlameDrawn:
.db 0
FuelEmpty:
.db 0
KeyFlag:
.db 0
Level:
.db 0
OffCourseStr:
.db "You  flew  off  course!",0
PressEnterStr:
.db "Press  Enter",0
LostStr:
.db "You  ran  out  of  lives!",0
LevelDoneStr:
.db "You  made  it!",0
BlewUpStr:
.db "You  blew  up!",0
BrianStr:
.db "                     "
.db "                     "
.db " briandm03@yahoo.com "
.db "                     "
.db "                     "
.db " Released: 08-21-08  ",0
Level1InfoStr:
.db "                     "
.db "   This moon has a   "
.db " gravity of 50. Its  "
.db " surface is flat so  "
.db "it should be easy to "
.db "      land on.       ",0
Level2InfoStr:
.db "                     "
.db "   This moon has a   "
.db " gravity of 100. Its "
.db "  surface is rocky.  "
.db " You will blow up if "
.db "you try to land going"
.db " faster than 20 mph. ",0
Level3InfoStr:
.db "                     "
.db "   This moon has a   "
.db " gravity of 150. It  "
.db "  has a mountainous  "
.db "surface so youll have"
.db "to find a flat space "
.db "     to land on.     ",0
Level4InfoStr:
.db "                     "
.db "   This moon has a   "
.db "gravity of 200. Youll"
.db " have to land in one "
.db " of the trenches to  "
.db "      make it.       ",0
Level5InfoStr:
.db "                     "
.db "   This moon has a   "
.db "gravity of 250. Youll"
.db " have to land in one "
.db "of the caves or else "
.db "    youll crash.     ",0
Level6InfoStr:
.db "                     "
.db "   This moon has a   "
.db "gravity of 300. Youll"
.db " have to find a flat "
.db " space to land in or "
.db "    youll crash.     ",0
EndGameStr:
.db "                     "
.db "                     "
.db "  Congratulations!   "
.db "                     "
.db "                     "
.db "      You Win!       ",0
Lander:
.db %00010000
.db %00101000
.db %01000100
.db %10000010
.db %10000010
.db %10000010
.db %01111100
.db %00101000
.db %00010000
.db %00111000
.db %01111100
.db %11111110
.db %11111110
.db %11111110
.db %01111100
.db %00101000

FlameUp1:
.db %00010000
.db %00000000
.db %00100000
.db %00000100
.db %00010000
.db %00000000
.db %01000000
.db %00001000
.db %00111000
.db %00111000
.db %01111100
.db %01111100
.db %11111110
.db %11111110
.db %01111100
.db %00111000

FlameUp2:
.db %00000000
.db %00101000
.db %00000000
.db %01000000
.db %00000100
.db %00010000
.db %00000000
.db %00100000
.db %00111000
.db %00111000
.db %01111100
.db %01111100
.db %11111110
.db %11111110
.db %01111100
.db %00111000

FlameDown1:
.db %00001000
.db %01000000
.db %00000000
.db %00010000
.db %00000100
.db %00100000
.db %00000000
.db %00010000
.db %00111000
.db %01111100
.db %11111110
.db %11111110
.db %01111100
.db %01111100
.db %00111000
.db %00111000

FlameDown2:
.db %00100000
.db %00000000
.db %00010000
.db %00000100
.db %01000000
.db %00000000
.db %00101000
.db %00000000
.db %00111000
.db %01111100
.db %11111110
.db %11111110
.db %01111100
.db %01111100
.db %00111000
.db %00111000

FlameLeft1:
.db %00000000
.db %00000000
.db %00010000
.db %00000001
.db %10001000
.db %00100000
.db %00000010
.db %00000000
.db %00000000
.db %00001100
.db %00111110
.db %11111111
.db %11111111
.db %11111111
.db %00111110
.db %00001100

FlameLeft2:
.db %00000000
.db %00000000
.db %00001000
.db %01000000
.db %00000100
.db %01000001
.db %00010000
.db %00000000
.db %00000000
.db %00001100
.db %00111110
.db %11111111
.db %11111111
.db %11111111
.db %00111110
.db %00001100

FlameRight1:
.db %00000000
.db %00000000
.db %01000000
.db %00000100
.db %00010001
.db %10000000
.db %00001000
.db %00000000
.db %00000000
.db %00110000
.db %01111100
.db %11111111
.db %11111111
.db %11111111
.db %01111100
.db %00110000

FlameRight2:
.db %00000000
.db %00000000
.db %00001000
.db %10000010
.db %00100000
.db %00000010
.db %00010000
.db %00000000
.db %00000000
.db %00110000
.db %01111100
.db %11111111
.db %11111111
.db %11111111
.db %01111100
.db %00110000

Explosion1:
.db %10000000
.db %00000100
.db %00100000
.db %10000000
.db %00010000
.db %00000100
.db %10000000
.db %00100100
.db %10000000
.db %00000100
.db %00100000
.db %10000000
.db %00010000
.db %00000100
.db %10000000
.db %00100100

Explosion2:
.db %01001000
.db %00000010
.db %01000000
.db %00010000
.db %00000010
.db %00001000
.db %01000000
.db %00000010
.db %01001000
.db %00000010
.db %01000000
.db %00010000
.db %00000010
.db %00001000
.db %01000000
.db %00000010


Stats:      ; 128 bytes
.db $20, $10, $00, $00, $00, $00, $10, $00, $00, $00, $02, $00, $00, $00, $00, $00
.db $50, $93, $01, $04, $40, $01, $13, $80, $00, $22, $22, $e0, $0c, $7f, $ff, $fe
.db $25, $54, $63, $4a, $a6, $99, $94, $29, $aa, $55, $52, $8a, $45, $7f, $ff, $fe
.db $71, $52, $55, $0a, $a5, $55, $55, $b2, $a8, $55, $52, $ca, $a4, $7f, $ff, $fe
.db $25, $51, $65, $4a, $a5, $59, $54, $a2, $aa, $55, $52, $8a, $c5, $7f, $ff, $fe
.db $50, $96, $43, $04, $44, $51, $53, $a1, $90, $22, $22, $8e, $64, $7f, $ff, $fe
.db $00, $10, $00, $00, $00, $00, $10, $00, $00, $00, $02, $00, $00, $00, $00, $00
.db $ff, $ff, $ff, $ff, $ff, $ff, $ff, $ff, $ff, $ff, $ff, $ff, $ff, $ff, $ff, $ff

FlatMoon:   ; 256 bytes
.db $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00
.db $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00
.db $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00
.db $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00
.db $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00
.db $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00
.db $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00
.db $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00
.db $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00
.db $07, $c0, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00
.db $3f, $ff, $c0, $00, $00, $00, $00, $00, $3f, $ff, $80, $00, $00, $00, $00, $00
.db $ff, $ff, $ff, $00, $00, $00, $00, $03, $ff, $ff, $fe, $00, $00, $00, $00, $03
.db $ff, $ff, $ff, $f0, $00, $00, $00, $1f, $ff, $ff, $ff, $f8, $00, $00, $00, $0f
.db $ff, $ff, $ff, $ff, $c0, $00, $00, $7f, $ff, $ff, $ff, $ff, $f0, $00, $00, $3f
.db $ff, $ff, $ff, $ff, $f0, $00, $07, $ff, $ff, $ff, $ff, $ff, $ff, $80, $00, $ff
.db $ff, $ff, $ff, $ff, $ff, $ff, $ff, $ff, $ff, $ff, $ff, $ff, $ff, $ff, $ff, $ff

RockyMoon:  ; 256 bytes
.db $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00
.db $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00
.db $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00
.db $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00
.db $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00
.db $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $0e
.db $00, $0e, $00, $00, $00, $00, $00, $07, $00, $00, $00, $00, $00, $00, $00, $1f
.db $00, $1f, $00, $00, $00, $00, $00, $0f, $80, $00, $00, $00, $00, $00, $00, $ff
.db $c0, $3f, $80, $00, $00, $00, $00, $ff, $c0, $00, $70, $00, $00, $00, $01, $ff
.db $e7, $ff, $80, $00, $00, $00, $01, $ff, $e0, $00, $f8, $00, $00, $00, $03, $ff
.db $ff, $ff, $c0, $00, $00, $00, $03, $ff, $ff, $ff, $fc, $03, $80, $00, $03, $ff
.db $ff, $ff, $ff, $30, $0e, $00, $03, $ff, $ff, $ff, $fe, $07, $c0, $38, $07, $ff
.db $ff, $ff, $ff, $f8, $1f, $00, $03, $ff, $ff, $ff, $ff, $ff, $e0, $7c, $6f, $ff
.db $ff, $ff, $ff, $ff, $ff, $80, $07, $ff, $ff, $ff, $ff, $ff, $f0, $fe, $ff, $ff
.db $ff, $ff, $ff, $ff, $ff, $c0, $0f, $ff, $ff, $ff, $ff, $ff, $ff, $ff, $ff, $ff
.db $ff, $ff, $ff, $ff, $ff, $ff, $ff, $ff, $ff, $ff, $ff, $ff, $ff, $ff, $ff, $ff

PointyMoon:     ; 256 bytes
.db $80, $00, $00, $00, $00, $00, $00, $00, $80, $00, $00, $00, $01, $00, $00, $00
.db $c0, $00, $00, $00, $00, $00, $00, $00, $80, $00, $00, $00, $01, $00, $00, $00
.db $c0, $00, $00, $00, $00, $00, $00, $01, $c0, $00, $00, $00, $03, $80, $00, $00
.db $e0, $00, $00, $00, $00, $00, $00, $01, $c0, $00, $00, $00, $03, $80, $00, $00
.db $e0, $00, $00, $00, $01, $00, $00, $01, $e0, $00, $00, $00, $07, $80, $00, $00
.db $f0, $00, $10, $00, $01, $80, $00, $03, $e0, $00, $40, $00, $07, $c0, $00, $04
.db $f0, $00, $10, $00, $03, $80, $00, $03, $e0, $00, $40, $00, $0f, $c0, $00, $04
.db $f8, $00, $38, $00, $03, $c0, $00, $03, $f0, $00, $e0, $00, $0f, $c0, $00, $0e
.db $fc, $00, $38, $00, $07, $c0, $00, $07, $f0, $00, $e0, $00, $1f, $e0, $00, $0e
.db $ff, $c0, $7c, $00, $07, $e0, $00, $07, $f8, $01, $f0, $00, $1f, $e0, $00, $1f
.db $ff, $ff, $fc, $00, $07, $e0, $00, $07, $ff, $ff, $f8, $00, $3f, $e0, $00, $1f
.db $ff, $ff, $ff, $00, $0f, $f0, $00, $0f, $ff, $ff, $fe, $00, $3f, $f0, $00, $3f
.db $ff, $ff, $ff, $f0, $1f, $f0, $00, $1f, $ff, $ff, $ff, $f8, $7f, $f0, $00, $3f
.db $ff, $ff, $ff, $ff, $ff, $f8, $00, $7f, $ff, $ff, $ff, $ff, $ff, $f0, $00, $7f
.db $ff, $ff, $ff, $ff, $ff, $fc, $07, $ff, $ff, $ff, $ff, $ff, $ff, $f8, $00, $ff
.db $ff, $ff, $ff, $ff, $ff, $ff, $ff, $ff, $ff, $ff, $ff, $ff, $ff, $ff, $ff, $ff

WaveMoon:   ; 256 bytes
.db $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00
.db $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00
.db $00, $00, $00, $00, $00, $03, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00
.db $00, $00, $00, $00, $00, $03, $80, $40, $00, $00, $00, $00, $70, $00, $f0, $00
.db $00, $00, $00, $1e, $00, $01, $e0, $c0, $03, $80, $00, $01, $e0, $00, $3c, $00
.db $00, $00, $00, $0f, $00, $00, $f1, $c0, $0f, $00, $70, $03, $c0, $00, $0e, $00
.db $00, $01, $80, $07, $c0, $00, $fb, $80, $1e, $00, $7c, $0f, $80, $00, $07, $80
.db $0c, $03, $00, $03, $e0, $00, $7f, $0c, $3c, $00, $1f, $1f, $00, $00, $03, $c0
.db $06, $0f, $00, $01, $f8, $00, $7f, $38, $38, $00, $0f, $fe, $00, $00, $01, $f0
.db $07, $1e, $00, $01, $fc, $00, $3f, $60, $78, $00, $07, $fe, $00, $00, $00, $f8
.db $03, $be, $00, $00, $fe, $00, $3f, $e0, $78, $00, $07, $fc, $00, $00, $00, $fc
.db $01, $fc, $00, $00, $ff, $00, $3f, $e0, $f8, $00, $03, $fc, $00, $00, $00, $7e
.db $81, $fc, $00, $00, $ff, $00, $3f, $ff, $fc, $00, $03, $fc, $00, $00, $00, $7e
.db $81, $f9, $c0, $01, $ff, $81, $ff, $ff, $ff, $80, $07, $fe, $f0, $7f, $f8, $ff
.db $c3, $ff, $fc, $03, $ff, $ff, $ff, $ff, $ff, $ff, $ff, $ff, $ff, $ff, $ff, $ff
.db $ff, $ff, $ff, $ff, $ff, $ff, $ff, $ff, $ff, $ff, $ff, $ff, $ff, $ff, $ff, $ff

TrenchMoon:     ;256 bytes
.db $00, $00, $00, $00, $10, $00, $00, $0a, $44, $00, $00, $22, $00, $00, $02, $00
.db $01, $40, $00, $0a, $38, $00, $02, $9f, $ee, $00, $00, $77, $00, $00, $07, $40
.db $03, $f0, $00, $1f, $7c, $00, $07, $ff, $ff, $00, $00, $ff, $90, $00, $1f, $e0
.db $07, $f8, $00, $3f, $fe, $00, $0f, $ff, $ff, $80, $01, $ff, $f8, $00, $3f, $f0
.db $0f, $fc, $00, $7f, $ff, $80, $3f, $ff, $ff, $c0, $03, $ff, $fc, $00, $7f, $fc
.db $1f, $fc, $00, $ff, $ff, $c0, $7f, $ff, $ff, $e0, $07, $ff, $fe, $00, $7f, $fe
.db $3f, $fe, $01, $ff, $ff, $c0, $7f, $ff, $ff, $f0, $0f, $ff, $ff, $00, $ff, $ff
.db $7f, $fe, $01, $ff, $ff, $e0, $ff, $ff, $ff, $f0, $0f, $ff, $ff, $00, $ff, $ff
.db $ff, $fe, $01, $ff, $ff, $f3, $ff, $ff, $ff, $f0, $0f, $ff, $ff, $00, $ff, $ff
.db $ff, $fe, $01, $ff, $ff, $ff, $ff, $ff, $ff, $f0, $0f, $ff, $ff, $00, $ff, $ff
.db $ff, $fe, $01, $ff, $ff, $ff, $ff, $ff, $ff, $f0, $0f, $ff, $ff, $00, $ff, $ff
.db $ff, $fe, $01, $ff, $ff, $ff, $ff, $ff, $ff, $f0, $0f, $ff, $ff, $00, $ff, $ff
.db $ff, $fe, $01, $ff, $ff, $ff, $ff, $ff, $ff, $f0, $0f, $ff, $ff, $00, $ff, $ff
.db $ff, $fe, $01, $ff, $ff, $ff, $ff, $ff, $ff, $f0, $0f, $ff, $ff, $00, $ff, $ff
.db $ff, $ff, $03, $ff, $ff, $ff, $ff, $ff, $ff, $f8, $1f, $ff, $ff, $81, $ff, $ff
.db $ff, $ff, $ff, $ff, $ff, $ff, $ff, $ff, $ff, $ff, $ff, $ff, $ff, $ff, $ff, $ff

CaveMoon:   ;256 bytes
.db $00, $04, $04, $4c, $00, $00, $00, $00, $00, $00, $00, $00, $00, $10, $09, $00
.db $00, $0e, $6e, $fe, $00, $00, $00, $00, $00, $00, $00, $00, $00, $7a, $3d, $a0
.db $00, $1f, $ff, $f8, $00, $00, $00, $00, $00, $00, $00, $00, $01, $ff, $7f, $f0
.db $00, $3f, $ff, $c0, $00, $00, $00, $00, $00, $00, $00, $00, $03, $ff, $ff, $fc
.db $01, $7f, $f8, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $1f, $ff, $fe
.db $03, $ff, $c0, $00, $00, $00, $00, $00, $00, $01, $4a, $00, $00, $00, $7f, $fe
.db $17, $ff, $00, $00, $00, $00, $29, $10, $00, $53, $ff, $00, $00, $00, $0f, $ff
.db $1f, $fe, $00, $00, $00, $02, $7f, $b8, $14, $ff, $ff, $80, $00, $00, $03, $ff
.db $3f, $f8, $00, $00, $00, $67, $ff, $fd, $3f, $ff, $ff, $80, $00, $00, $01, $ff
.db $7f, $f0, $00, $00, $00, $ff, $ff, $ff, $ff, $ff, $ff, $c0, $00, $00, $00, $ff
.db $ff, $f0, $00, $00, $00, $ff, $ff, $ff, $ff, $ff, $ff, $d0, $00, $00, $00, $7f
.db $ff, $f0, $00, $00, $05, $ff, $ff, $ff, $ff, $ff, $ff, $f0, $00, $00, $00, $7f
.db $ff, $f0, $00, $00, $2f, $ff, $ff, $ff, $ff, $ff, $ff, $fa, $00, $00, $00, $7f
.db $ff, $f8, $00, $00, $7f, $ff, $ff, $ff, $ff, $ff, $ff, $fe, $80, $00, $00, $ff
.db $ff, $fe, $00, $01, $ff, $ff, $ff, $ff, $ff, $ff, $ff, $ff, $e0, $00, $01, $ff
.db $ff, $ff, $ff, $ff, $ff, $ff, $ff, $ff, $ff, $ff, $ff, $ff, $ff, $ff, $ff, $ff

Banner:     ; 256 bytes
.db $00, $0e, $0e, $00, $00, $00, $00, $00, $30, $00, $00, $00, $01, $80, $00, $00
.db $00, $1e, $1e, $00, $00, $00, $00, $00, $70, $00, $00, $00, $03, $00, $00, $00
.db $00, $1e, $1e, $00, $00, $00, $00, $00, $60, $00, $00, $00, $03, $00, $00, $00
.db $00, $1e, $3c, $00, $00, $00, $00, $00, $60, $00, $00, $00, $03, $00, $00, $00
.db $00, $36, $6c, $00, $00, $00, $00, $00, $e0, $00, $00, $00, $06, $00, $00, $00
.db $00, $36, $dc, $3f, $07, $e1, $bc, $00, $c0, $3f, $0d, $e0, $fe, $1f, $86, $e0
.db $00, $36, $d8, $77, $0e, $e1, $fc, $00, $c0, $3f, $0f, $e1, $fe, $3b, $8f, $c0
.db $00, $67, $98, $e3, $9c, $73, $8c, $01, $c0, $43, $1c, $63, $8c, $71, $8e, $00
.db $00, $67, $38, $c3, $98, $73, $0c, $01, $80, $03, $18, $67, $0c, $61, $8c, $00
.db $00, $66, $31, $83, $30, $63, $1c, $01, $80, $3f, $18, $e6, $0c, $ff, $9c, $00
.db $00, $c6, $31, $83, $30, $66, $18, $03, $80, $fe, $30, $c6, $18, $c0, $18, $00
.db $00, $c4, $73, $87, $70, $e6, $18, $03, $01, $c6, $30, $ce, $19, $c0, $18, $00
.db $00, $c0, $63, $86, $70, $c6, $38, $03, $01, $8e, $31, $cc, $39, $c1, $38, $00
.db $01, $80, $63, $8c, $71, $8c, $30, $07, $03, $8c, $61, $8e, $71, $c2, $30, $00
.db $01, $80, $e3, $fc, $7f, $8c, $30, $07, $ff, $fe, $61, $8f, $f1, $fe, $30, $00
.db $03, $80, $c1, $f0, $3e, $1c, $70, $0f, $f9, $ee, $e3, $87, $b0, $f8, $70, $00

IntCopy:
.org IntAddress
IntStart:
 ex af,af'
 exx
 ld a,%01111101
 out (1),a
 nop
 in a,(1)
 rla
 rla
 rla
 rla
 and %00010000
 ld b,a
 ld a, %01111110
 out (1), a
 nop
 in a, (1)
 and %00001111
 or b
 ld b,a
 ld a, %00111111
 out (1), a
 nop
 in a, (1)
 and %01000000
 or b
 res 7,a
 ld (KeyFlag),a
 ld a,(FrameTimer)
 dec a
 ld (FrameTimer),a
 jp nz,EndInt
 ld a,(Frame)
 xor 255
 ld (Frame),a
 ld a,10
 ld (FrameTimer),a
EndInt:
 in a,(3)
 rra
 ld a,0
 adc a,9
 out (3),a
 ld a,$0B
 out (3),a
 ei
 ex af,af'
 exx
 reti
IntEnd
.end

